import { defineStore } from "pinia";
import type { User } from "@/client/auth";
import { UserService, RoleEnum } from "@/client/auth";
import { S3KeyService } from "@/client/s3proxy";
import type { S3Key } from "@/client/s3proxy";
import { OpenAPI as S3ProxyOpenAPI } from "@/client/s3proxy";
import { OpenAPI as AuthOpenAPI } from "@/client/auth";
import { OpenAPI as WorkflowOpenAPI } from "@/client/workflow";

type DecodedToken = {
  exp: number;
  iss: string;
  roles: RoleEnum[];
  sub: string;
};

export type RootState = {
  token: string | null;
  decodedToken: DecodedToken | null;
  user: User | null;
  s3key: S3Key | null;
};

function parseJwt(token: string): DecodedToken {
  const base64Url = token.split(".")[1];
  const base64 = base64Url.replace(/-/g, "+").replace(/_/g, "/");
  const jsonPayload = decodeURIComponent(
    window
      .atob(base64)
      .split("")
      .map(function (c) {
        return "%" + ("00" + c.charCodeAt(0).toString(16)).slice(-2);
      })
      .join(""),
  );

  return JSON.parse(jsonPayload) as DecodedToken;
}

export const useAuthStore = defineStore({
  id: "auth",
  state: () =>
    ({
      token: null,
      decodedToken: null,
      user: null,
      s3key: null,
    }) as RootState,
  getters: {
    roles(): string[] {
      return (
        this.user?.roles?.map((role) => role.toString()) ??
        this.decodedToken?.["roles"] ??
        []
      );
    },
    authenticated: (state) => state.token != null,
    currentUID(): string {
      return this.decodedToken?.["sub"] ?? "";
    },
    foreignUser: (state) =>
      state.user?.roles?.includes(RoleEnum.FOREIGN_USER) ??
      state.decodedToken?.roles.includes(RoleEnum.FOREIGN_USER) ??
      true,
    normalUser: (state) =>
      state.user?.roles?.includes(RoleEnum.USER) ??
      state.decodedToken?.roles.includes(RoleEnum.USER) ??
      false,
    workflowReviewer: (state) =>
      state.user?.roles?.includes(RoleEnum.REVIEWER) ??
      state.decodedToken?.roles.includes(RoleEnum.REVIEWER) ??
      false,
    workflowDev: (state) =>
      state.user?.roles?.includes(RoleEnum.DEVELOPER) ??
      state.decodedToken?.roles.includes(RoleEnum.DEVELOPER) ??
      false,
    admin: (state) =>
      state.user?.roles?.includes(RoleEnum.ADMINISTRATOR) ??
      state.decodedToken?.roles.includes(RoleEnum.ADMINISTRATOR) ??
      false,
  },
  actions: {
    setToken(token: string | null) {
      if (token != null) {
        this.token = token;
        this.decodedToken = parseJwt(token);
        S3ProxyOpenAPI.TOKEN = token;
        AuthOpenAPI.TOKEN = token;
        WorkflowOpenAPI.TOKEN = token;
        UserService.userGetLoggedInUser()
          .then((user) => {
            this.updateUser(user);
          })
          .catch(() => {
            this.token = null;
          });
      } else {
        this.token = null;
        this.decodedToken = null;
        this.user = null;
      }
    },
    setS3Key(key: S3Key | null) {
      this.s3key = key;
    },
    updateUser(user: User) {
      this.user = user;
      S3KeyService.s3KeyGetUserKeys(user.uid)
        .then((keys) => {
          if (keys.length > 0) {
            this.setS3Key(keys[0]);
          } else {
            this.setS3Key(null);
          }
        })
        .catch(() => {
          this.setS3Key(null);
        });
    },
    logout() {
      this.setToken(null);
    },
  },
});
